package de.hdm_stuttgart.mi.sd1.figuretest;

import org.junit.Assert;
import org.junit.Test;

import de.hdm_stuttgart.mi.sd1.figure.model.Circle;
import de.hdm_stuttgart.mi.sd1.figure.model.Rectangle;

public class FigureTest {
   static final double accuracyLimit = 1.E-15;  // Rounding accuracy limit with respect to arithmetic operations
   
   @Test
   public void circleTranslation() {
      
      final double      
             x = 3.,               y =  4.,         // circle's center coordinates (3, 4)
        transX = 3.,          transY = -2.,         // translation vector (3, -2)
       expectX = x + transX, expectY =  y + transY; // translated circle's coordinates (6, 2)
      
      final Circle c = new Circle(x, y, 3.0);
      c.move(transX, transY);
      
      Assert.assertEquals(expectX, c.getX(), accuracyLimit);
      Assert.assertEquals(expectY, c.getY(), accuracyLimit);
   }

   @Test
   public void rectangleTranslation() {
      
      final double
               x = 2.,               y =  6.5,       // rectangle's center coordinates (2, 6.5)
          transX = 4.,          transY =  1.,        // translation vector (3, -2)
         expectX = x + transX, expectY = y + transY; // translated circle's coordinates (6, 2)
      
      final Rectangle r = new Rectangle(x, y, 2., 1.);
      r.move(transX, transY);
      Assert.assertEquals(expectX, r.getX(), accuracyLimit);
      Assert.assertEquals(expectY, r.getY(), accuracyLimit);
   }

   @Test
   public void circlePerimeterAndArea() {
      final double
                    radius = 1.,
         expectedPerimeter = 2 * Math.PI * radius,
              expectedArea = Math.PI * radius * radius;
      
      final Circle c = new Circle(3, 4, radius);

      Assert.assertEquals(expectedPerimeter, c.getPerimeter(), accuracyLimit);
      Assert.assertEquals(expectedArea, c.getArea(), accuracyLimit);
   }

   @Test
   public void rectanglePerimeterAndArea() {
      final double
                     width = 2.,
                    height = 1.,
         expectedPerimeter = 2 * (width + height),
              expectedArea = width * height;
      
      final Rectangle r = new Rectangle(1,  3, width, height);
      
      Assert.assertEquals(expectedPerimeter, r.getPerimeter(), accuracyLimit);
      Assert.assertEquals(expectedArea, r.getArea(), accuracyLimit);
   }
   
   @Test
   public void rectangleScale() {
      final double
          width = 2.,
         height = 1.,
         factor = 3;
      
      final Rectangle r = new Rectangle(1,  3, width, height);
      
      final double
        originalPerimeter = r.getPerimeter(),
             originalArea = r.getArea(),
             expectedPerimeter = originalPerimeter * factor, // Perimeter is linear with respect to scale factor
             expectedArea = originalArea * factor * factor;  // Area is quadratic with respect to scale factor
      
      r.scale(factor);
      
      Assert.assertEquals(expectedPerimeter, r.getPerimeter(), accuracyLimit);
      Assert.assertEquals(expectedArea, r.getArea(), accuracyLimit);
   }

   @Test
   public void circleScale() {
      final double
         radius = 2.,
         factor = 0.7;
      
      final Circle c = new Circle(-2,  5, radius);
      
      final double
        originalPerimeter = c.getPerimeter(),
             originalArea = c.getArea(),
             expectedPerimeter = originalPerimeter * factor, // Perimeter is linear with respect to scale factor
             expectedArea = originalArea * factor * factor;  // Area is quadratic with respect to scale factor
      
      c.scale(factor);
      
      Assert.assertEquals(expectedPerimeter, c.getPerimeter(), accuracyLimit);
      Assert.assertEquals(expectedArea, c.getArea(), accuracyLimit);
   }
}
