package de.hdm_stuttgart.mi.sd1.figure.model;

/**
 * 
 */
public abstract class Figure {
     
   private double x,y;
   
   
   
   @Override
   public String toString() {
      return "(" + x + ',' + y + ')';
   }

   /**
    * Creating a figure with given center coordinates
    *  
    * @param x
    * @param y
    */
   public Figure(final double x, final double y) {
      setX(x);
      setY(y);
   }
   
   // To be implemented in derived classes Rectangle and Circle
   /**
    * 
    * @param factor Scale the current figure by this value.
    * @return The current object.
    */
   public abstract Figure scale(double factor);

   /**
    * 
    * @return The current figure's area e.g. with times height in case of a rectangle
    */
   public abstract double getArea();
   
   /**
    * @return The current figure's perimeter e.g. with 2 * (height + width) in case of a rectangle
    */
   public abstract double getPerimeter();
   
   public double getX() { return x;}
   public void setX(final double x) { this.x = x;}
   public double getY() { return y;}
   public void setY(final double y) { this.y = y;}
   
   /**
    * Translating a figure by a given vector (x,y).
    * 
    * @param x
    * @param y
    * @return The current object.
    */
   public final Figure move(final double x, final double y) {
      setX(getX() + x);
      setY(getY() + y);
      return this;
   }   
}